﻿using SecureShopFileImplement.Models;
using System.Xml.Linq;

namespace SecureShopFileImplement
{
    public class DataFileSingleton
    {
        private static DataFileSingleton? instance;
        private readonly string FacilitiesFileName = "Facilities.xml";
        private readonly string OrderFileName = "Order.xml";
        private readonly string SecureFileName = "Secure.xml";
        public List<Facilities> Facilitiess { get; private set; }
        private readonly string ClientFileName = "Client.xml";
        private readonly string MessageInfoFileName = "MessageInfo.xml";
        public List<Order> Orders { get; private set; }
        public List<Secure> Secures { get; private set; }
        public List<Client> Clients { get; private set; }
        public List<MessageInfo> MessageInfos { get; private set; }

        public static DataFileSingleton GetInstance()
        {
            if (instance == null)
            {
                instance = new DataFileSingleton();
            }
            return instance;
        }

        public void SaveFacilitiess() => SaveData(Facilitiess, FacilitiesFileName, "Facilitiess", x => x.GetXElement);
        public void SaveSecures() => SaveData(Secures, SecureFileName, "Secures", x => x.GetXElement);
        public void SaveOrders() => SaveData(Orders, OrderFileName, "Orders", x => x.GetXElement);
        public void SaveClients() => SaveData(Clients, ClientFileName, "Clients", x => x.GetXElement);
        public void SaveMessageInfos() => SaveData(MessageInfos, MessageInfoFileName, "MessageInfos", x => x.GetXElement);

        private DataFileSingleton()
        {
            Facilitiess = LoadData(FacilitiesFileName, "Facilities", x => Facilities.Create(x)!)!;
            Secures = LoadData(SecureFileName, "Secure", x => Secure.Create(x)!)!;
            Orders = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
            Clients = LoadData(ClientFileName, "Client", x => Client.Create(x)!)!;
            MessageInfos = LoadData(MessageInfoFileName, "MessageInfo", x => MessageInfo.Create(x)!)!;
        }
        private static List<T>? LoadData<T>(string filename, string xmlNodeName, Func<XElement, T> selectFunction)
        {
            if (File.Exists(filename))
            {
                return XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList();
            }
            return new List<T>();
        }
        private static void SaveData<T>(List<T> data, string filename, string xmlNodeName, Func<T, XElement> selectFunction)
        {
            if (data != null)
            {
                new XDocument(new XElement(xmlNodeName, data.Select(selectFunction).ToArray())).Save(filename);
            }
        }
    }
}