﻿using SecureShopContracts.BindingModels;
using SecureShopContracts.ViewModels;
using SecureShopDataModels.Models;
using System.Xml.Linq;

namespace SecureShopFileImplement.Models
{
    public class MessageInfo : IMessageInfoModel
    {
        public string MessageId { get; set; } = string.Empty;

        public int? ClientId { get; set; }

        public string SenderName { get; set; } = string.Empty;

        public DateTime DateDelivery { get; set; } = DateTime.Now;

        public string Subject { get; set; } = string.Empty;

        public string Body { get; set; } = string.Empty;

        public static MessageInfo? Create(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new MessageInfo()
            {
                MessageId = model.MessageId,
                ClientId = model.ClientId,
                SenderName = model.SenderName,
                DateDelivery = model.DateDelivery,
                Subject = model.Subject,
                Body = model.Body
            };
        }

        public static MessageInfo? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new MessageInfo()
            {
                MessageId = element.Element("MessageId")!.Value,
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                SenderName = element.Element("MessageId")!.Value,
                DateDelivery = DateTime.ParseExact(element.Element("DateDelivery")!.Value, "G", null),
                Subject = element.Element("MessageId")!.Value,
                Body = element.Element("MessageId")!.Value
            };
        }
        public MessageInfoViewModel GetViewModel => new()
        {
            MessageId = MessageId,
            ClientId = ClientId,
            SenderName = SenderName,
            DateDelivery = DateDelivery,
            Subject = Subject,
            Body = Body
        };

        public XElement GetXElement => new("MessageInfo", new XAttribute("MessageId", MessageId), new XElement("ClientId", ClientId), new XElement("SenderName", SenderName), new XElement("DateDelivery", DateDelivery), new XElement("Subject", Subject), new XElement("Body", Body));
    }
}